/*
 * Copyright 2017 NXP
 * All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

/*
 * TEXT BELOW IS USED AS SETTING FOR TOOLS *************************************
!!GlobalInfo
product: Pins v3.0
processor: MK28FN2M0xxx15
package_id: MK28FN2M0VMI15
mcu_data: ksdk2_0
processor_version: 2.0.0
board: FRDM-K28F
 * BE CAREFUL MODIFYING THIS COMMENT - IT IS YAML SETTINGS FOR TOOLS ***********
 */

#include "fsl_common.h"
#include "fsl_port.h"
#include "pin_mux.h"



#define PIN14_IDX                       14u   /*!< Pin number for pin 14 in a port */
#define PIN15_IDX                       15u   /*!< Pin number for pin 15 in a port */
#define PIN16_IDX                       16u   /*!< Pin number for pin 16 in a port */
#define PIN17_IDX                       17u   /*!< Pin number for pin 17 in a port */
#define PIN24_IDX                       24u   /*!< Pin number for pin 24 in a port */
#define PIN25_IDX                       25u   /*!< Pin number for pin 25 in a port */
#define SOPT5_LPUART0RXSRC_LPUART_RX  0x00u   /*!< LPUART0 receive data source select: LPUART0_RX pin */
#define SOPT5_LPUART0TXSRC_LPUART_TX  0x00u   /*!< LPUART0 transmit data source select: LPUART0_TX pin */

/*
 * TEXT BELOW IS USED AS SETTING FOR TOOLS *************************************
BOARD_InitPins:
- options: {callFromInitBoot: 'true', prefix: BOARD_, coreID: core0, enableClock: 'true'}
- pin_list:
  - {pin_num: A7, peripheral: LPUART0, signal: RX, pin_signal: PTC25/LPUART0_RX/FB_A4/SDRAM_D4/QSPI0A_SCLK}
  - {pin_num: B7, peripheral: LPUART0, signal: TX, pin_signal: PTC24/LPUART0_TX/FB_A5/SDRAM_D5/QSPI0A_DATA3}
  - {pin_num: K13, peripheral: SPI0, signal: PCS0_SS, pin_signal: PTA14/SPI0_PCS0/LPUART0_TX/TRACE_D2/FXIO0_D20/I2S0_RX_BCLK/I2S0_TXD1}
  - {pin_num: K12, peripheral: SPI0, signal: SCK, pin_signal: PTA15/SPI0_SCK/LPUART0_RX/TRACE_D1/FXIO0_D21/I2S0_RXD0}
  - {pin_num: J13, peripheral: SPI0, signal: SOUT, pin_signal: PTA16/SPI0_SOUT/LPUART0_CTS_b/TRACE_D0/FXIO0_D22/I2S0_RX_FS/I2S0_RXD1}
  - {pin_num: J12, peripheral: SPI0, signal: SIN, pin_signal: PTA17/SPI0_SIN/LPUART0_RTS_b/FXIO0_D23/I2S0_MCLK/I2S1_MCLK}
 * BE CAREFUL MODIFYING THIS COMMENT - IT IS YAML SETTINGS FOR TOOLS ***********
 */

/*FUNCTION**********************************************************************
 *
 * Function Name : BOARD_InitPins
 * Description   : Configures pin routing and optionally pin electrical features.
 *
 *END**************************************************************************/
void BOARD_InitPins(void) {
  CLOCK_EnableClock(kCLOCK_PortA);                           /* Port A Clock Gate Control: Clock enabled */
  CLOCK_EnableClock(kCLOCK_PortC);                           /* Port C Clock Gate Control: Clock enabled */

  PORT_SetPinMux(PORTA, PIN14_IDX, kPORT_MuxAlt2);           /* PORTA14 (pin K13) is configured as SPI0_PCS0 */
  PORT_SetPinMux(PORTA, PIN15_IDX, kPORT_MuxAlt2);           /* PORTA15 (pin K12) is configured as SPI0_SCK */
  PORT_SetPinMux(PORTA, PIN16_IDX, kPORT_MuxAlt2);           /* PORTA16 (pin J13) is configured as SPI0_SOUT */
  PORT_SetPinMux(PORTA, PIN17_IDX, kPORT_MuxAlt2);           /* PORTA17 (pin J12) is configured as SPI0_SIN */
  PORT_SetPinMux(PORTC, PIN24_IDX, kPORT_MuxAlt3);           /* PORTC24 (pin B7) is configured as LPUART0_TX */
  PORT_SetPinMux(PORTC, PIN25_IDX, kPORT_MuxAlt3);           /* PORTC25 (pin A7) is configured as LPUART0_RX */
  SIM->SOPT5 = ((SIM->SOPT5 &
    (~(SIM_SOPT5_LPUART0TXSRC_MASK | SIM_SOPT5_LPUART0RXSRC_MASK))) /* Mask bits to zero which are setting */
      | SIM_SOPT5_LPUART0TXSRC(SOPT5_LPUART0TXSRC_LPUART_TX) /* LPUART0 transmit data source select: LPUART0_TX pin */
      | SIM_SOPT5_LPUART0RXSRC(SOPT5_LPUART0RXSRC_LPUART_RX) /* LPUART0 receive data source select: LPUART0_RX pin */
    );
}

/*******************************************************************************
 * EOF
 ******************************************************************************/
/*
 * Copyright 2023 NXP
 * All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

void _Connect(void)
{
  int value;
 
  JLINK_CORESIGHT_WriteDP(2, 0x00000000);
  value = JLINK_CORESIGHT_ReadAP(0);

  JLINK_SYS_Report1("*** CSW: ", value);

  // Config Debug Mailbox when cannot connect
  if((value & 0x40) == 0)
  {
    JLINK_SYS_Report("*** ConfigDebugMailbox start");

    // Read AP ID register to identify DM AP at index 1
    JLINK_CORESIGHT_WriteDP(2, 0x020000f0);
    // The returned AP ID should be 0x002A0000
    value = JLINK_CORESIGHT_ReadAP(3);
    JLINK_SYS_Report1("AP ID:", value);

    // Select DM AP index 2
    JLINK_CORESIGHT_WriteDP(2, 0x02000000);
    JLINK_CORESIGHT_ReadDP(0);

    // Active DebugMailbox, Write DM RESYNC_REQ + CHIP_RESET_REQ
    JLINK_CORESIGHT_WriteAP(0, 0x21);
    // Poll CSW register (0) for zero return, indicating success
    value = -1;
    while (value != 0)
    {
      value = JLINK_CORESIGHT_ReadAP(0);
    }
    JLINK_SYS_Report1("RESYNC_REQ + CHIP_RESET_REQ:", value);

    // Start DM-AP
    JLINK_CORESIGHT_WriteAP(1, 0x01);
    // Poll RETURN register (2) for zero return
    value = -1;
    while (value != 0)
    {
      value = JLINK_CORESIGHT_ReadAP(2) & 0xFFFF;
    }
    JLINK_SYS_Report1("Start DM-AP:", value);

    // Enter Debug Session,  Write DM START_DBG_SESSION to REQUEST register (1)
    JLINK_CORESIGHT_WriteAP(1, 0x07);
    // Poll RETURN register (2) for zero return
    value = -1;
    while (value != 0)
    {
      value = JLINK_CORESIGHT_ReadAP(2) & 0xFFFF;
    }
    JLINK_SYS_Report1("DEBUG_SESSION_REQ:", value);

    JLINK_SYS_Report("*** ConfigDebugMailbox end");
  }
}

void AfterResetTarget(void)
{
  // Clear XPSR to avoid undefined instruction fault caused by IT/ICI
  JLINK_MEM_WriteU32(0xE000EDF8, 0x01000000);
  JLINK_MEM_WriteU32(0xE000EDF4, 0x00010010);
  JLINK_SYS_Sleep(1);
  // Clear MSPLIM to 0
  JLINK_MEM_WriteU32(0xE000EDF8, 0x00000000);
  JLINK_MEM_WriteU32(0xE000EDF4, 0x0001001C);
  JLINK_SYS_Sleep(1);
  // Disable MPU
  JLINK_MEM_WriteU32(0xE000ED94, 0x00000000);
}

/*********************************************************************
*
*       ResetTarget
*/
void ResetTarget(void) {
  int tmp;

  JLINK_SYS_Report("*** JLINK ResetTarget");
  // Halt the core
  JLINK_MEM_WriteU32(0xE000EDF0, 0xA05F0003);

  // Clear VECTOR CATCH and set TRCENA
  tmp = JLINK_MEM_ReadU32(0xE000EDFC) & ~1;
  JLINK_MEM_WriteU32(0xE000EDFC, tmp | (1<<24));

  // Breakoint at SYSCON_BASE + 0x40
  JLINK_MEM_WriteU32(0xE0001020, 0x50000040);
  JLINK_MEM_WriteU32(0xE0001028, 0x814);

  // Reset using the SYSRESETREQ
  JLINK_SYS_Report("Set CM33_SYSRESETREQ_EN");
  JLINK_MEM_WriteU32(0xE000ED0C, 0x5FA0004);
  JLINK_SYS_Sleep(50);

  _Connect();

  // Halt the core in case it didn't stop at a breakpiont
  JLINK_MEM_WriteU32(0xE000EDF0, 0xA05F0003);

  // Clear the Breakpoints
  JLINK_MEM_WriteU32(0xE0001020, 0x0);
  JLINK_MEM_WriteU32(0xE0001028, 0x0);
}

/*********************************************************************
*
*       InitTarget
*/
void InitTarget(void)
{

  JLINK_SYS_Report("******************************************************");
  JLINK_SYS_Report("J-Link script: MCX Cortex-M33 core J-Link script");
  JLINK_SYS_Report("******************************************************");
  JLINK_CORESIGHT_Configure("IRPre=0;DRPre=0;IRPost=0;DRPost=0;IRLenDevice=4");
  CPU = CORTEX_M33;      // Pre-select that we have a Cortex-M33 connected
  JTAG_AllowTAPReset = 0;   // J-Link is allowed to use a TAP reset for JTAG-chain auto-detection

  JTAG_SetDeviceId(0, 0x6BA02477);  // 4-bits IRLen

  _Connect();

  CORESIGHT_IndexAHBAPToUse = 0;
}
